'use strict';

exports.__esModule = true;
exports.default = dateSort;

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _mixed = require('../../../helpers/mixed');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Date sorting algorithm
 *
 * @param {Boolean} sortOrder Sorting order (`true` for ascending, `false` for descending).
 * @param {Object} columnMeta Column meta object.
 * @returns {Function} The compare function.
 */
function dateSort(sortOrder, columnMeta) {
  return function (a, b) {
    if (a[1] === b[1]) {
      return 0;
    }

    if ((0, _mixed.isEmpty)(a[1])) {
      if ((0, _mixed.isEmpty)(b[1])) {
        return 0;
      }

      if (columnMeta.columnSorting.sortEmptyCells) {
        return sortOrder ? -1 : 1;
      }

      return 1;
    }

    if ((0, _mixed.isEmpty)(b[1])) {
      if ((0, _mixed.isEmpty)(a[1])) {
        return 0;
      }

      if (columnMeta.columnSorting.sortEmptyCells) {
        return sortOrder ? 1 : -1;
      }

      return -1;
    }

    var aDate = (0, _moment2.default)(a[1], columnMeta.dateFormat);
    var bDate = (0, _moment2.default)(b[1], columnMeta.dateFormat);

    if (!aDate.isValid()) {
      return 1;
    }
    if (!bDate.isValid()) {
      return -1;
    }

    if (bDate.isAfter(aDate)) {
      return sortOrder ? -1 : 1;
    }
    if (bDate.isBefore(aDate)) {
      return sortOrder ? 1 : -1;
    }

    return 0;
  };
}