'use strict';

exports.__esModule = true;
exports.default = numericSort;

var _mixed = require('../../../helpers/mixed');

/**
 * Numeric sorting algorithm.
 *
 * @param {Boolean} sortOrder Sorting order (`true` for ascending, `false` for descending).
 * @param {Object} columnMeta Column meta object.
 * @returns {Function} The compare function.
 */
function numericSort(sortOrder, columnMeta) {
  return function (a, b) {
    var parsedA = parseFloat(a[1]);
    var parsedB = parseFloat(b[1]);

    // Watch out when changing this part of code!
    // Check below returns 0 (as expected) when comparing empty string, null, undefined
    if (parsedA === parsedB || isNaN(parsedA) && isNaN(parsedB)) {
      return 0;
    }

    if (columnMeta.columnSorting.sortEmptyCells) {
      if ((0, _mixed.isEmpty)(a[1])) {
        return sortOrder ? -1 : 1;
      }

      if ((0, _mixed.isEmpty)(b[1])) {
        return sortOrder ? 1 : -1;
      }
    }

    if (isNaN(parsedA)) {
      return 1;
    }

    if (isNaN(parsedB)) {
      return -1;
    }

    if (parsedA < parsedB) {
      return sortOrder ? -1 : 1;
    } else if (parsedA > parsedB) {
      return sortOrder ? 1 : -1;
    }

    return 0;
  };
}